summaryrefslogtreecommitdiffstats
path: root/HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c
diff options
context:
space:
mode:
Diffstat (limited to 'HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c')
-rw-r--r--HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c110
1 files changed, 95 insertions, 15 deletions
diff --git a/HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c b/HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c
index 0cac86b..dc82568 100644
--- a/HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c
+++ b/HAL-afb/HAL_MOST_UNICENS/wrap_unicens.c
@@ -21,14 +21,18 @@
#include <string.h>
#include <json-c/json.h>
#include <afb/afb-binding.h>
-
#include "wrap_unicens.h"
#include "wrap-json.h"
typedef struct async_job_ {
- wrap_ucs_result_cb_t result_fptr;
+ wrap_ucs_result_cb_t result_fptr;
void *result_user_ptr;
-} async_job_t;
+} async_job_t;
+
+typedef struct parse_result_ {
+ int done;
+ char *str_result;
+} parse_result_t;
/* Subscribes to unicens2-binding events.
* \return Returns 0 if successful, otherwise != 0".
@@ -63,19 +67,78 @@ OnErrorExit:
return err;
}
+/* Checks if name ends with a given letter. */
+static int wrap_ucs_string_ends_with(const char *name, char letter) {
+
+ int result = 0;
+ size_t len = strlen(name);
+
+ if (len > 0) {
+
+ if (name[len] == letter) {
+ result = 1;
+ }
+ }
+
+ return result;
+}
+
+/* Callback for iteration through found files. Marks search as "done" as
+ * soon as the search pattern "kit.xml" matches.
+ * \param closure User reference. Points to parse_result_t.
+ * \param j_obj Points to json object within array.
+ */
+static void wrap_ucs_find_xml_cb(void *closure, struct json_object *j_obj) {
+
+ const char *dir, *name;
+ parse_result_t *result;
+
+ if (!closure)
+ return;
+
+ result = (parse_result_t *)closure;
+ if (result->done)
+ return;
+
+ wrap_json_unpack(j_obj, "{s:s, s:s}", "dirpath", &dir, "basename", &name);
+ AFB_DEBUG("Found file: %s", name);
+
+ if(strstr(name, "kit.xml") != NULL) {
+ size_t sz = strlen(dir)+strlen(name)+10;
+ char * full_path = malloc(sz);
+
+ strncpy(full_path, dir, sz);
+
+ if (!wrap_ucs_string_ends_with(dir, '/'))
+ strncat(full_path, "/", sz);
+
+ strncat(full_path, name, sz);
+ AFB_DEBUG("Found XML file: %s", full_path);
+
+ result->done = 1;
+ result->str_result = full_path;
+ }
+}
+
/* Retrieves XML configuration path to initialize unicens2-binding.
* \param config_path Directory containing XML configuration file(s).
* \return Returns 0 if successful, otherwise != 0".
*/
-extern int wrap_ucs_getconfig_sync(const char *config_path) {
- json_object *j_response, *j_query = NULL;
- int err;
+extern int wrap_ucs_getconfig_sync(const char *config_path, char **file_found) {
+
+ int err = 0;
+ json_object *j_response, *j_query, *j_paths = NULL;
+ parse_result_t result = {.done = 0, .str_result = NULL};
+
+ *file_found = NULL;
/* Build JSON object to retrieve UNICENS configuration */
- if (config_path != NULL)
- err = wrap_json_pack(&j_query, "{s:s}", "cfgpath", config_path);
- else
+ if ((config_path == NULL) || (strcmp(config_path, "") == 0))
err = wrap_json_pack(&j_query, "{}");
+ else
+ err = wrap_json_pack(&j_query, "{s:s}", "cfgpath", config_path);
if (err) {
AFB_ERROR("Failed to create listconfig json object");
@@ -88,7 +151,25 @@ extern int wrap_ucs_getconfig_sync(const char *config_path) {
goto OnErrorExit;
}
else {
- AFB_NOTICE("UNICENS listconfig result, res=%s", json_object_to_json_string(j_response));
+ AFB_DEBUG("UNICENS listconfig result, res=%s", json_object_to_json_string(j_response));
+
+ if (json_object_object_get_ex(j_response, "response", &j_paths)) {
+
+ AFB_DEBUG("UNICENS listconfig result, paths=%s", json_object_to_json_string(j_paths));
+
+ wrap_json_optarray_for_all(j_paths, &wrap_ucs_find_xml_cb, &result);
+
+ if (result.done) {
+ *file_found = strdup(result.str_result);
+ free(result.str_result);
+ result.str_result = NULL;
+ }
+ else {
+ AFB_NOTICE("wrap_ucs_getconfig_sync, search pattern does not match for paths=%s", json_object_to_json_string(j_paths));
+ err = -1;
+ }
+ }
+
json_object_put(j_response);
}
json_object_put(j_query);
@@ -100,7 +181,6 @@ OnErrorExit:
return err;
}
-
/* Initializes the unicens2-binding.
* \param file_name Path to XML configuration file or \c NULL for
* first found file in default path.
@@ -170,11 +250,11 @@ extern int wrap_ucs_i2cwrite_sync(uint16_t node, uint8_t *data_ptr, uint8_t data
err = afb_service_call_sync("UNICENS", "writei2c", j_query, &j_response);
if (err) {
- AFB_ERROR("Failed to call writei2c");
+ AFB_ERROR("Failed to call writei2c_sync");
goto OnErrorExit;
}
else {
- AFB_NOTICE("Called writei2c, res=%s", "async"/*json_object_to_json_string(j_response)*/);
+ AFB_INFO("Called writei2c_sync, res=%s", json_object_to_json_string(j_response));
json_object_put(j_response);
}
@@ -191,7 +271,7 @@ OnErrorExit:
static void wrap_ucs_i2cwrite_cb(void *closure, int status, struct json_object *j_result) {
- AFB_DEBUG("wrap_ucs_i2cwrite_cb: closure=%p status=%d, res=%s", closure, status, json_object_to_json_string(j_result));
+ AFB_INFO("wrap_ucs_i2cwrite_cb: closure=%p status=%d, res=%s", closure, status, json_object_to_json_string(j_result));
if (closure) {
async_job_t *job_ptr = (async_job_t *)closure;
@@ -63,19 +67,78 @@ OnErrorExit:
/* Retrieves XML configuration path to initialize unicens2-binding.
- * \param config_path Directory containing XML configuration file(s).
+ * \param config_path Directory containing XML configuration file(s).
+ * \param file_found Points to found file_path or NULL if not found.
+ * Memory must be released by calling free().
* \return Returns 0 if successful, otherwise != 0".
*/